package PureMessage::Database::PostgreSQL::Service;
use strict;
use warnings;
use Carp;
use PureMessage::Version qw($PREFIX);
use PureMessage::Config qw(%config);
use PureMessage::_sys ();
use POSIX ();

use base 'Exporter';
our @EXPORT = qw(
    db_start
    db_stop
    db_restart
    db_status
    db_analyze
    db_vacuum
    db_print_status
    db_dump
    db_import
    db_version
);

my $pgctl = "$PREFIX/bin/pmx-database-pg";

# exec pg_ctl under a new SID so that it doesn't
# pick up signals from the admin's shell/console
sub _system_setsid ($;@) {
    my $prog = shift;
    my $pid = fork;
    if ($pid) {
        my $r = waitpid($pid,0);
        if ($r == $pid) {
            return ($? >> 8);
        } else {
	    warn "Error waiting for '$prog' to finish: $!\n";
	    return -1;
	}
    }
    elsif (defined($pid)) {
        PureMessage::_sys::setsid();
        open(STDIN, '</dev/null');
        exec($prog, @_); 
	warn "Can't exec '$prog': $!\n";
	POSIX::_exit(-1);
    }
    else {
	warn "Can't fork & exec '$prog': $!\n";
	return -1;
    }
}

sub db_start {
    return _system_setsid($pgctl,'start',@_);
}

sub db_stop {
    return system($pgctl, "stop", @_) >> 8;
}

sub db_restart {
    return _system_setsid($pgctl,'restart',@_);
}

# Gather the status from pmx-database-pg and return as a hash
sub db_status {
    my %info;
    local $_;
    if (-e $pgctl) {
        open(my $PG, "$pgctl fullstatus |") or die "Can't open \"$pgctl fullstatus|\": $!";
        while(<$PG>) {
            if (/^\s*(\w+)\s+=\s+(\S+)$/) {
                $info{$1} = $2;
            }
        }
    }
    else {
        # We're not configured (pg_ctl doesn't exist!)
        $info{port} = 0;
        $info{pid} = 0;
        $info{running} = 0;
        $info{hostname} = PureMessage::_sys::gethostname();
    }
    return \%info;
}

sub db_print_status {
    my $status = system($pgctl, "status", @_) >> 8;
    if ($status == 0) {
        my $prefix = $config{prefix};
        my $cfg_file = "$prefix/postgres/etc/database.conf";
        if (-e $cfg_file) {
            my $parser = ActiveState::Config->new();
            $parser->parse_file($cfg_file);
            my %pg_opts = $parser->as_hash;
            my $db_dir = $pg_opts{data_dir};
            if (my $disk_util = `du -ks $db_dir`) {
                $disk_util =~ s/([0-9]+).*/disk usage: $1/;
                print $disk_util;
                $status = 0;
            }
        }
    } elsif($status == 1) {
        # postgres returns status '1' for not running, we want that to be 2
        $status = 2;
    }

    return $status;
}

sub db_analyze {
    return system($pgctl, "analyze", @_) >> 8;
}

sub db_vacuum {
    return system($pgctl, "vacuum", @_) >> 8;
}

sub db_dump {
    return system($pgctl, "dump", @_) >> 8;
}

sub db_import {
    return system($pgctl, "import", @_) >> 8;
}

sub db_version {
    my $ver = `$pgctl version`;
    chomp($ver);
    return $ver;
}

1;
